# Generated by Django 3.2.9 on 2025-02-08 13:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_url_group_permissions', '0001_initial'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='groupurlpermissions',
            name='django_url__url_7c741b_idx',
        ),
        migrations.RemoveIndex(
            model_name='groupurlpermissions',
            name='django_url__http_me_61e08d_idx',
        ),
        migrations.AddIndex(
            model_name='groupurlpermissions',
            index=models.Index(fields=['url'], name='django_url__url_ba8d57_idx'),
        ),
        migrations.AddIndex(
            model_name='groupurlpermissions',
            index=models.Index(fields=['http_method'], name='django_url__http_me_43a03c_idx'),
        ),
    ]



# Copyright (c) 2025 IT ELAZOS SL
# Intellectual property of IT ELAZOS SL. All rights reserved.
#
# This source code is part of the "django_url_group_permissions" project.
# Unauthorized use, reproduction, distribution, modification, or transmission
# of this code, in whole or in part, is strictly prohibited without the express
# written consent of IT ELAZOS SL.
#
# Unauthorized use of this code will be considered an infringement of
# intellectual property rights and may result in legal action.

from django.apps import AppConfig


class DjangoUrlPermissionsConfig(AppConfig):
    name = 'django_url_group_permissions'
    verbose_name = "Django URL Permissions"

    def ready(self):
        #add here setup for entire app. Executes only once in runserver
        pass      
        





# Copyright (c) 2024 IT ELAZOS SL
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# This source code is part of the "django_url_group_permissions" project.
# Intellectual property of IT ELAZOS SL.

from django.contrib import admin
from django.urls import URLPattern, URLResolver
from django.conf import settings
from .models import *
from django.contrib.auth.admin import GroupAdmin
from django.contrib.auth.models import Group
from django.contrib import admin
from .models import GroupUrlPermissions
from django.utils.translation import gettext_lazy as _


class CustomGroupAdmin(GroupAdmin):
    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        return form

    def get_clean_url(self, url_path):
        """Remove language prefix if present and clean the URL path."""
        if hasattr(settings, 'LANGUAGES'):
            for lang_code, _ in settings.LANGUAGES:
                prefix = f'{lang_code}/'
                if url_path.startswith(prefix):
                    return url_path[len(prefix):]
        return url_path

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        
        # Get current group
        group = self.get_object(request, object_id)
        
        if group:
            # Get chosen URL permissions
            chosen_url_permissions = GroupUrlPermissions.objects.filter(group=group)
            
            # Get HTTP methods from model
            http_methods = [method[0] for method in GroupUrlPermissions.HTTP_METHODS]
            
            # Get all URLs using the recursive method
            available_urls = set()
            urlconf = __import__(settings.ROOT_URLCONF, {}, {}, [''])
            
            def list_urls(patterns, acc=None):
                if acc is None:
                    acc = []
                if not patterns:
                    return
                pattern = patterns[0]
                if isinstance(pattern, URLPattern):
                    url_path = ''.join(acc + [str(pattern.pattern)])
                    # Clean URL path
                    clean_url = self.get_clean_url(url_path)
                    # Add all HTTP methods for user-defined URLs
                    for method in http_methods:
                        available_urls.add((clean_url, method))
                            
                elif isinstance(pattern, URLResolver):
                    yield from list_urls(pattern.url_patterns, acc + [str(pattern.pattern)])
                yield from list_urls(patterns[1:], acc)
            
            # Process all URL patterns
            for _ in list_urls(urlconf.urlpatterns):
                pass
            
            # Convert set to list of dictionaries
            available_urls = [
                {
                    'id': f"{url}|{method}",
                    'url': url,
                    'http_method': method,
                }
                for url, method in available_urls
            ]
            
            # Filter out URLs that are already chosen
            chosen_url_methods = []
            for perm in chosen_url_permissions:
                clean_url = self.get_clean_url(perm.url)
                chosen_url_methods.append({
                    'id': f"{clean_url}|{perm.http_method}",
                    'url': clean_url,
                    'http_method': perm.http_method,
                })       

            # Create a set of chosen IDs for efficient lookup
            chosen_ids = {item['id'] for item in chosen_url_methods}

            available_url_permissions = [
                url for url in available_urls 
                if f"{url['url']}|{url['http_method']}" not in chosen_ids
            ]
            
            extra_context.update({
                'available_url_permissions': available_url_permissions,
                'chosen_url_permissions': chosen_url_methods
            })
        
        return super().change_view(
            request, object_id, form_url, extra_context=extra_context,
        )
    
    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)
        
        # Clear all existing URL permissions for this group
        GroupUrlPermissions.objects.filter(group=obj).delete()
        
        # get the list of url and method from the form
        chosen_permissions = request.POST.getlist('url_permissions')

        if chosen_permissions:
            for perm_id in chosen_permissions:
                if perm_id:  # Only process non-empty values
                    url, method = perm_id.split('|')
                    # Store URL without language prefix
                    clean_url = self.get_clean_url(url)
                    GroupUrlPermissions.objects.create(
                        group=obj,
                        url=clean_url,
                        http_method=method
                    )
                

# Unregister the default GroupAdmin
admin.site.unregister(Group)
# Register the custom GroupAdmin
admin.site.register(Group, CustomGroupAdmin) 


@admin.register(GroupUrlPermissions)
class GroupUrlPermissionsAdmin(admin.ModelAdmin):
    list_display = ('group', 'url', 'http_method', 'is_active', 'created_at')
    list_filter = ('group', 'http_method', 'is_active')
    search_fields = ('group__name', 'url', 'description')
    ordering = ('group__name', 'url')
    readonly_fields = ('created_at', 'updated_at')
    
    fieldsets = (
        (None, {
            'fields': ('group', 'url', 'http_method')
        }),
        (_('Options'), {
            'fields': ('is_active', 'description')
        }),
        (_('Timestamps'), {
            'fields': ('created_at', 'updated_at'),
            'classes': ('collapse',)
        }),
    )
















# Generated by Django 3.2.9 on 2025-02-07 13:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupUrlPermissions',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('url', models.CharField(help_text='URL pattern (can include wildcards like /api/*/users/)', max_length=255)),
                ('http_method', models.CharField(choices=[('GET', 'GET'), ('POST', 'POST'), ('PUT', 'PUT'), ('PATCH', 'PATCH'), ('DELETE', 'DELETE'), ('HEAD', 'HEAD'), ('OPTIONS', 'OPTIONS'), ('ALL', 'ALL')], default='GET', help_text='HTTP method this permission applies to', max_length=10)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this permission is currently active')),
                ('description', models.TextField(blank=True, help_text='Optional description of what this permission does', null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='url_permissions', to='auth.group')),
            ],
            options={
                'verbose_name': 'URL Permission',
                'verbose_name_plural': 'URL Permissions',
                'ordering': ('group__name', 'url'),
            },
        ),
        migrations.AddIndex(
            model_name='groupurlpermissions',
            index=models.Index(fields=['url'], name='django_url__url_7c741b_idx'),
        ),
        migrations.AddIndex(
            model_name='groupurlpermissions',
            index=models.Index(fields=['http_method'], name='django_url__http_me_61e08d_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='groupurlpermissions',
            unique_together={('group', 'url', 'http_method')},
        ),
    ]



# Copyright (c) 2024 IT ELAZOS SL
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# This source code is part of the "django_url_group_permissions" project.
# Intellectual property of IT ELAZOS SL.

from django.db import models
from django.contrib.auth.models import Group
from django.utils.translation import gettext_lazy as _


class GroupUrlPermissions(models.Model):
    HTTP_METHODS = (
        ('GET', 'GET'),
        ('POST', 'POST'),
        ('PUT', 'PUT'),
        ('PATCH', 'PATCH'),
        ('DELETE', 'DELETE'),
        ('HEAD', 'HEAD'),
        ('OPTIONS', 'OPTIONS'),
        ('ALL', 'ALL'),
    )

    id = models.AutoField(primary_key=True)
    group = models.ForeignKey(
        Group,
        on_delete=models.CASCADE,
        null=False,
        blank=False,
        related_name='url_permissions'
    )
    url = models.CharField(
        max_length=255,
        null=False,
        blank=False,
        help_text=_('URL pattern (can include wildcards like /api/*/users/)')
    )
    http_method = models.CharField(
        max_length=10,
        choices=HTTP_METHODS,
        default='GET',
        help_text=_('HTTP method this permission applies to')
    )
    is_active = models.BooleanField(
        default=True,
        help_text=_('Whether this permission is currently active')
    )
    description = models.TextField(
        null=True,
        blank=True,
        help_text=_('Optional description of what this permission does')
    )
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ('group__name', 'url')
        verbose_name_plural = _('URL Permissions')
        verbose_name = _('URL Permission')
        unique_together = ('group', 'url', 'http_method')
        indexes = [
            models.Index(fields=['url']),
            models.Index(fields=['http_method']),
        ]

    def __str__(self):
        return f"{self.group.name} - {self.http_method} {self.url}"

    @classmethod
    def has_url_permission(cls, user, url, method='GET'):
        """
        Check if a user has permission to access a specific URL with given method.
        """        

        if url.startswith('/'):
            url = url[1:]

        if user.is_superuser:
            return True
            
        user_groups = user.groups.all()
        return cls.objects.filter(
            group__in=user_groups,
            url=url,
            is_active=True,
        ).filter(
            models.Q(http_method=method.upper()) | 
            models.Q(http_method='ALL')
        ).exists()




# Copyright (c) 2024 IT ELAZOS SL
# All rights reserved.
#
# MIT License
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# This source code is part of the "django_url_group_permissions" project.
# Intellectual property of IT ELAZOS SL.

from django.http import HttpResponseForbidden
from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from .models.models import GroupUrlPermissions
from django.urls import get_resolver, is_valid_path
from django.utils.translation import get_language


class UrlPermissionMiddleware(MiddlewareMixin):
    """
    Middleware to check if the user's groups have permission to access the current URL.
    """
    
    def __init__(self, get_response):
        super().__init__(get_response)
        self.exempt_urls = getattr(settings, 'URL_PERMISSION_EXEMPT_URLS', [])
        self.permission_required = getattr(settings, 'URL_PERMISSION_REQUIRED', True)
        self.check_all_views = getattr(settings, 'URL_PERMISSION_CHECK_ALL_VIEWS', False)

    def is_exempt_url(self, path):
        """Check if the URL is exempt from permission checks."""
        return any(path.startswith(exempt_url) for exempt_url in self.exempt_urls)

    def get_path_without_language(self, path):
        """Remove language prefix from path if it exists."""
        if hasattr(settings, 'LANGUAGES'):
            current_language = get_language()
            if current_language:
                language_prefix = f'/{current_language}/'
                if path.startswith(language_prefix):
                    return path[len(language_prefix)-1:]
        return path

    def process_request(self, request):
        # Get path without language prefix for permission checking
        path_to_check = self.get_path_without_language(request.path)
        
        # Skip permission check for exempt URLs        
        if self.is_exempt_url(path_to_check):
            return None
    
        # Skip permission check if URL_PERMISSION_REQUIRED is False
        if not self.permission_required:
            return None

        # Always allow authentication-related URLs
        if not request.user.is_authenticated:
            return None
        
        # Skip permission check for superusers
        if request.user.is_superuser:
            return None

        # Get the view function
        resolver = get_resolver()
        try:
            resolver_match = resolver.resolve(path_to_check)
            view_func = resolver_match.func
        except:
            # If resolution fails with the modified path, try original path
            resolver_match = resolver.resolve(request.path)
            view_func = resolver_match.func

        # Check if permissions should be checked
        check_permissions = (
            self.check_all_views or 
            getattr(view_func, 'requires_url_permission', False)
        )
        
        if not check_permissions:
            return None

        # Get the current HTTP method
        method = request.method

        # Check if user has permission for this URL
        has_permission = GroupUrlPermissions.has_url_permission(
            user=request.user,
            url=path_to_check,  # Use the path without language prefix
            method=method
        )

        if not has_permission:
            return HttpResponseForbidden("You don't have permission to access this URL.")

        return None



from setuptools import setup, find_packages

# Read the README file for the long description
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="django_url_group_permissions",  # Replace with your package name
    version="1.0.2",  # Initial version
    author="Elazos",  # Your name or organization
    author_email="info@elazos.com",  # Your email
    description="A Django package for managing URL-based permissions through user groups with HTTP method support",
    long_description=long_description,  # Long description from README
    long_description_content_type="text/markdown",  # Format of the long description
    url="https://github.com/jmarxuach/django-url-group-permissions",  # Project URL
    project_urls={
        "Bug Tracker": "https://github.com/jmarxuach/django-url-group-permissions/issues",  # Issue tracker
    },
    classifiers=[
        "Development Status :: 3 - Alpha",  # Package maturity
        "Framework :: Django",
        "Framework :: Django :: 3.2",  # Django version compatibility
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",  # License
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Security",
    ],
    packages=find_packages(),  # Automatically find packages in the directory
    include_package_data=True,  # Include non-Python files (e.g., templates, static files)
    python_requires=">=3.7",  # Python version requirement
    install_requires=[
        "Django>=3.2",  # Django dependency
    ],
    extras_require={
        "dev": [
            "black",  # Code formatting
            "flake8",  # Linting
            "pytest",  # Testing
            "pytest-django",  # Django testing
        ],
    },
)